unit hiImg_Gradient;

interface

uses Windows,Kol,Share,Debug,Messages;

type
  PColor = ^TColor;

{$I share.inc}
type
  TGradientStyle  = (SingleVert, SingleHoriz, SingleLeft, SingleRight, DoubleVert, DoubleHoriz, DoubleLeft, DoubleRight, AngleLeftTop, AngleLeftBott, AngleRightTop, AngleRightBott, Center);
type
  ThiImg_Gradient = class(TDebug)
   private
    x1,y1,x2,y2:integer; 
    fFrame: boolean;
    fGradient: boolean;
    fInversGrad:boolean;
    fStartColor:TColor;
    fEndColor:TColor;
    fFrameColor:TColor;
    fGradientStyle:TGradientStyle;
   public
    _prop_Point1:integer;
    _prop_Point2:integer;
    _prop_Point2AsOffset:boolean;
    _prop_DrawSource: procedure(var _Data:TData) of object;
    
    _event_onDraw:THI_Event;
    _data_Bitmap:THI_Event;
    _data_Point1:THI_Event;
    _data_Point2:THI_Event;

    property _prop_GradientStyle : TGradientStyle write fGradientStyle;
    property _prop_Frame         : boolean write fFrame;
    property _prop_Gradient      : boolean write fGradient;
    property _prop_InversGrad    : boolean write fInversGrad;
    property _prop_StartColor    : integer write fStartColor;
    property _prop_EndColor      : integer write fEndColor;
    property _prop_FrameColor    : integer write fFrameColor;

    procedure _work_doDraw(var _Data:TData; Index:word);

    procedure _work_doGradientStyle(var _Data:TData; Index:word);
    procedure _work_doFrame(var _Data:TData; Index:word);
    procedure _work_doGradient(var _Data:TData; Index:word);
    procedure _work_doInversGrad(var _Data:TData; Index:word);
    procedure _work_doStartColor(var _Data:TData; Index:word);
    procedure _work_doEndColor(var _Data:TData; Index:word);
    procedure _work_doFrameColor(var _Data:TData; Index:word);

    procedure Bitmap(var _Data:TData);
    procedure Handle(var _Data:TData);
  end;

implementation

//******************************  **************************************

type
  COLOR16 = $0000..$FF00;
  TTriVertex = packed record
    x, y: DWORD;
    Red, Green, Blue, Alpha: COLOR16;
  end;
function GradientFill(DC: HDC; Vertex: Pointer; NumVertex: Cardinal;
                      Mesh: Pointer; NumMesh, Mode: DWORD): BOOL; stdcall;
                      external 'msimg32.dll' name 'GradientFill';

procedure _Gradient(DC:HDC; cbRect:TRect; Gradient:boolean; StartColor,EndColor:TColor; InversGrad:boolean; GradientStyle:TGradientStyle);
var   EColor: TRGB;
      SColor: TRGB;
      Color:TColor;
      hdcMem:HDC;
      hdcBmp:HBITMAP;
      Nvert, NgRect:integer;
      vert: array[0..4] of TTriVertex;
      gRect: array[0..1] of TGradientRect;
      gTri: array[0..3] of TGradientTriangle;
begin
   if (GradientStyle = SingleRight) or (GradientStyle = DoubleRight) then InversGrad := not InversGrad;
   if Gradient then begin
      if InversGrad then begin
         Color      := EndColor;
         EndColor   := StartColor;
         StartColor := Color;
      end;
   end else if InversGrad then
      EndColor := StartColor      
   else
      StartColor := EndColor;

   PColor(@SColor)^:= Color2RGB(StartColor);
   PColor(@EColor)^:= Color2RGB(EndColor);

   if (GradientStyle = SingleVert) or (GradientStyle = SingleHoriz) or (GradientStyle = DoubleVert) or (GradientStyle = DoubleHoriz) then begin
      vert[0].x := cbRect.Left;
      vert[0].y := cbRect.Top;
      if (GradientStyle = DoubleHoriz) or (GradientStyle = DoubleVert) then begin
         if GradientStyle = DoubleVert then begin
            vert[1].x := cbRect.Right;
            vert[1].y := (cbRect.Bottom + cbRect.Top) div 2;
            vert[2].x := cbRect.Left;
            vert[2].y := (cbRect.Bottom + cbRect.Top) div 2;
         end else begin
            vert[1].x := (cbRect.Right + cbRect.Left) div 2;
            vert[1].y := cbRect.Bottom;
            vert[2].x := (cbRect.Right + cbRect.Left) div 2;
            vert[2].y := cbRect.Top;
         end;
         vert[3].x     := cbRect.Right;
         vert[3].y     := cbRect.Bottom;
         vert[2].Red   := EColor.R shl 8;
         vert[2].Green := EColor.G shl 8;
         vert[2].Blue  := EColor.B shl 8;
         vert[2].Alpha := $0000;
         vert[3].Red   := SColor.R shl 8;
         vert[3].Green := SColor.G shl 8;
         vert[3].Blue  := SColor.B shl 8;
         vert[3].Alpha := $0000;         
         gRect[1].UpperLeft  := 2;
         gRect[1].LowerRight := 3;
         Nvert := 4;
         NgRect := 2;
      end else begin  
         vert[1].x      := cbRect.Right;
         vert[1].y      := cbRect.Bottom;
         Nvert := 2;
         NgRect := 1;
      end;   

      vert[0].Red    := SColor.R shl 8;
      vert[0].Green  := SColor.G shl 8;
      vert[0].Blue   := SColor.B shl 8;
      vert[0].Alpha  := $0000;
      vert[1].Red    := EColor.R shl 8;
      vert[1].Green  := EColor.G shl 8;
      vert[1].Blue   := EColor.B shl 8;
      vert[1].Alpha  := $0000;
      gRect[0].UpperLeft  := 0;
      gRect[0].LowerRight := 1;

      if (GradientStyle = SingleVert) or (GradientStyle = DoubleVert) then
         GradientFill(DC, @vert, Nvert, @gRect, NgRect, GRADIENT_FILL_RECT_V)
      else
         GradientFill(DC, @vert, Nvert, @gRect, NgRect, GRADIENT_FILL_RECT_H);

   end else begin

      vert[0].x     := cbRect.Left;
      vert[0].y     := cbRect.Top;
      if (GradientStyle = AngleRightTop) or (GradientStyle = AngleLeftBott)  or (GradientStyle = AngleRightBott) then begin
         vert[0].Red   := EColor.R shl 8;
         vert[0].Green := EColor.G shl 8;
         vert[0].Blue  := EColor.B shl 8;
      end else begin
         vert[0].Red   := SColor.R shl 8;
         vert[0].Green := SColor.G shl 8;
         vert[0].Blue  := SColor.B shl 8;
      end;
      vert[0].Alpha := $0000;
      
      vert[1].x     := cbRect.Right;
      vert[1].y     := cbRect.Top;
      if (GradientStyle = Center) or (GradientStyle = AngleRightTop) then begin 
         vert[1].Red   := SColor.R shl 8;
         vert[1].Green := SColor.G shl 8;
         vert[1].Blue  := SColor.B shl 8;
      end else begin
         vert[1].Red   := EColor.R shl 8;
         vert[1].Green := EColor.G shl 8;
         vert[1].Blue  := EColor.B shl 8;
      end;
      vert[1].Alpha := $0000;

      vert[2].x     := cbRect.Left;
      vert[2].y     := cbRect.Bottom;
      if (GradientStyle = Center) or (GradientStyle = AngleLeftBott) then begin
         vert[2].Red   := SColor.R shl 8;
         vert[2].Green := SColor.G shl 8;
         vert[2].Blue  := SColor.B shl 8;
      end else begin
         vert[2].Red   := EColor.R shl 8;
         vert[2].Green := EColor.G shl 8;
         vert[2].Blue  := EColor.B shl 8;
      end;
      vert[2].Alpha := $0000;

      vert[3].x     := cbRect.Right;
      vert[3].y     := cbRect.Bottom;
      if (GradientStyle = AngleRightTop) or (GradientStyle = AngleLeftTop) or (GradientStyle = AngleLeftBott) then begin
         vert[3].Red   := EColor.R shl 8;
         vert[3].Green := EColor.G shl 8;
         vert[3].Blue  := EColor.B shl 8;
      end else begin
         vert[3].Red   := SColor.R shl 8;
         vert[3].Green := SColor.G shl 8;
         vert[3].Blue  := SColor.B shl 8;
      end;
      vert[3].Alpha := $0000;
  
      vert[4].x     := (cbRect.Right + cbRect.Left) div 2;
      vert[4].y     := (cbRect.Bottom + cbRect.Top) div 2;
      vert[4].Red   := EColor.R shl 8;
      vert[4].Green := EColor.G shl 8;
      vert[4].Blue  := EColor.B shl 8;
      vert[4].Alpha := $0000;

      if (GradientStyle = SingleRight) or (GradientStyle = SingleLeft) then begin
         vert[1].x     := vert[1].x * 2;
         vert[2].y     := vert[2].y * 2;
         vert[3].x     := vert[3].x * 2;
         vert[3].y     := vert[3].y * 2;                      
      end;

      if GradientStyle = Center then begin
         gTri[0].Vertex1 := 0;
         gTri[0].Vertex2 := 1;
         gTri[0].Vertex3 := 4;
         gTri[1].Vertex1 := 1;
         gTri[1].Vertex2 := 3;
         gTri[1].Vertex3 := 4;
         gTri[2].Vertex1 := 2;
         gTri[2].Vertex2 := 3;
         gTri[2].Vertex3 := 4;
         gTri[3].Vertex1 := 2;
         gTri[3].Vertex2 := 0;
         gTri[3].Vertex3 := 4;
      end else if (GradientStyle = AngleRightTop) or (GradientStyle = AngleLeftTop) then begin
            gTri[0].Vertex2 := 2;
            gTri[0].Vertex3 := 3;
            gTri[1].Vertex2 := 0;
            gTri[1].Vertex3 := 1;
            if GradientStyle = AngleLeftTop then begin
               gTri[0].Vertex1 := 0;
               gTri[1].Vertex1 := 3;
            end else begin  
               gTri[0].Vertex1 := 1;
               gTri[1].Vertex1 := 2;
            end;
      end else if (GradientStyle = AngleRightBott) or (GradientStyle = AngleLeftBott) then begin
               gTri[0].Vertex2 := 0;
               gTri[0].Vertex3 := 1;
               gTri[1].Vertex1 := 2;
               gTri[1].Vertex2 := 3;
            if GradientStyle = AngleLeftBott then begin
               gTri[0].Vertex1 := 2;
               gTri[1].Vertex3 := 1;
            end else begin  
               gTri[0].Vertex1 := 3;
               gTri[1].Vertex3 := 0;
            end;
      end else begin
         gTri[0].Vertex1 := 0;
         gTri[0].Vertex2 := 1;
         gTri[1].Vertex1 := 3;
         gTri[1].Vertex3 := 2;
         if (GradientStyle = SingleLeft) or (GradientStyle = DoubleLeft) then begin
            gTri[0].Vertex3 := 2;
            gTri[1].Vertex2 := 1;
         end else begin  
            gTri[0].Vertex3 := 3;
            gTri[1].Vertex2 := 0;
         end;
      end;

      if GradientStyle = Center then
         GradientFill(DC, @vert, 5, @gTri, 4, GRADIENT_FILL_TRIANGLE)
      else if (GradientStyle = SingleRight) or (GradientStyle = SingleLeft) then begin
         hdcMem:= CreateCompatibleDC(0);
         hdcBmp:= CreateCompatibleBitmap(DC,vert[1].x-vert[0].x, vert[2].y-vert[0].y);
         SelectObject(hdcMem, hdcBmp);         
         GradientFill(hdcMem, @vert, 4, @gTri, 1, GRADIENT_FILL_TRIANGLE);
         if GradientStyle = SingleLeft then
            BitBlt(DC, cbRect.Left, cbRect.Top, cbRect.Right-cbRect.Left, cbRect.Bottom-cbRect.Top, hdcMem, vert[0].x, vert[0].y, SRCCOPY)
         else
            BitBlt(DC, cbRect.Left, cbRect.Top, cbRect.Right-cbRect.Left, cbRect.Bottom-cbRect.Top, hdcMem, vert[1].x div 2, vert[1].y, SRCCOPY);         
         DeleteDC(hdcMem);
         DeleteObject(hdcBmp);        
      end else          
         GradientFill(DC, @vert, 4, @gTri, 2, GRADIENT_FILL_TRIANGLE)
   end;      
end;

//******************************************************************************

procedure ThiImg_Gradient._work_doDraw;
begin
   _prop_DrawSource(_Data);
end;

procedure ThiImg_Gradient.Bitmap;
var   p: dword;
      dt:TData;
      bmp:PBitmap;
begin
   dt := _Data;
   bmp := ReadBitmap(_Data,_data_Bitmap,nil);
   if (bmp = nil) and (bmp.Empty) then exit;
   
   p  := ReadInteger(_Data, _data_Point1, _prop_Point1);
   y1 := p shr 16;
   x1 := p and $FFFF;
   p  := ReadInteger(_Data,_data_Point2,_prop_Point2);
   if p <> 0 then begin
      y2 := p shr 16;
      x2 := p and $FFFF;
   end;
   if _prop_Point2AsOffset then begin
      inc(x2,x1);
      inc(y2,y1);
   end;

   with Bmp.Canvas{$ifndef F_P}^{$endif} do begin
      _Gradient(Handle, PRect(@x1)^, fGradient, fStartColor, fEndColor, fInversGrad, fGradientStyle);
      Pen.Color := fFrameColor;
      if fFrame then begin
         Brush.BrushStyle := bsClear;
         Rectangle(x1,y1,x2,y2);
      end;   
   end;
   _hi_CreateEvent(_Data,@_event_onDraw,dt);
end;

procedure ThiImg_Gradient.Handle;
var   p: dword;
      DC:HDC;
      wnd:HWND;
      br:HBRUSH;
      pen:HPEN;      
      dt:TData;
begin
   dt := _Data;
   wnd := ReadInteger(_Data,_data_Bitmap,0);
   DC := GetDC(wnd);
     
   p  := ReadInteger(_Data, _data_Point1, _prop_Point1);
   y1 := p shr 16;
   x1 := p and $FFFF;
   p  := ReadInteger(_Data,_data_Point2,_prop_Point2);
   if p <> 0 then begin
      y2 := p shr 16;
      x2 := p and $FFFF;
   end;
   if _prop_Point2AsOffset then begin
      inc(x2,x1);
      inc(y2,y1);
   end;
   _Gradient(DC, PRect(@x1)^, fGradient, fStartColor, fEndColor, fInversGrad, fGradientStyle);
   if fFrame then begin
      pen := CreatePen(0,1,fFrameColor);
      br := GetStockObject(NULL_BRUSH);
      SelectObject(DC,br);
      SelectObject(DC,Pen);
      Windows.Rectangle(DC,x1,y1,x2,y2);
      DeleteObject(br);
      DeleteObject(pen);
   end;
   ReleaseDC(Wnd,DC);
   _hi_CreateEvent(_Data,@_event_onDraw,dt);
end;

procedure ThiImg_Gradient._work_doGradientStyle;begin fGradientStyle := TGradientStyle(ToInteger(_Data));end;
procedure ThiImg_Gradient._work_doFrame;begin fFrame := ReadBool(_Data);end;
procedure ThiImg_Gradient._work_doGradient;begin fGradient := ReadBool(_Data);end;
procedure ThiImg_Gradient._work_doInversGrad;begin fInversGrad := ReadBool(_Data);end;
procedure ThiImg_Gradient._work_doStartColor;begin fStartColor := ToInteger(_Data);end;
procedure ThiImg_Gradient._work_doEndColor;begin fEndColor := ToInteger(_Data);end;
procedure ThiImg_Gradient._work_doFrameColor;begin fFrameColor := ToInteger(_Data);end;

end.